#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <cstdlib>
#include <ctime>

const int WIDTH = 640;
const int HEIGHT = 480;
const int MARGIN = 40;
const double UPDATE_INTERVAL = 0.1;  // Update every second

class Graph : public Fl_Box {
public:
    Graph(int x, int y, int w, int h, const char* label = 0)
        : Fl_Box(x, y, w, h, label), num_points(100) {
        srand(time(0));
        generateData();
    }

    void generateData() {
        int max_data = h() - 2 * MARGIN;
        for (int i = 0; i < num_points; ++i) {
            data[i] = MARGIN + (rand() % max_data);
        }
    }

    void update() {
        generateData();  // Generate new random data
        redraw();        // Redraw the graph
    }

    void draw() override {
        Fl_Box::draw();
        fl_color(FL_BLACK);
        fl_line_style(FL_SOLID, 2);
        drawAxes();
        drawData();
    }

private:
    int num_points;
    int data[100];

    void drawAxes() {
        fl_line(MARGIN, MARGIN, MARGIN, h() - MARGIN);           // Y-axis
        fl_line(MARGIN, h() - MARGIN, w() - MARGIN, h() - MARGIN); // X-axis
    }

    void drawData() {
    
        int step_x = (w() - 2 * MARGIN) / (num_points - 1);
            // Clear the old data by filling the area with a white background
        fl_color(FL_WHITE);
        fl_rectf(x(), y(), w(), h());  // Clear the drawing area
        
        fl_color(FL_BLUE);
        fl_line_style(FL_SOLID, 1);

        for (int i = 0; i < num_points - 1; ++i) {
            int x1 = MARGIN + i * step_x;
            int y1 = data[i];
            int x2 = MARGIN + (i + 1) * step_x;
            int y2 = data[i + 1];

            fl_line(x1, y1, x2, y2);
        }

        for (int i = 0; i < num_points; ++i) {
            int x = MARGIN + i * step_x;
            int y = data[i];
            fl_color(FL_GREEN);
            int diameter=3;
            fl_pie(x - 3, y - 3, diameter, diameter, 0, 360);  // Draw circles at points
        }
    }
};

// Function that gets called every second to update the graph
void TimerCallback(void* graph) {
    Graph* g = (Graph*)graph;
    g->update();   // Update the graph with new data

    // Re-register the timer to call this function again after 1 second
    Fl::repeat_timeout(UPDATE_INTERVAL, TimerCallback, graph);
}

int main() {
    Fl_Window window(WIDTH, HEIGHT, "Dynamic FLTK Graph");
    Graph graph(MARGIN, MARGIN, WIDTH - 2 * MARGIN, HEIGHT - 2 * MARGIN, "Graph");

    window.end();
    window.resizable(window);
    window.show();

    // Add the timer callback to update the graph every second
    Fl::add_timeout(UPDATE_INTERVAL, TimerCallback, &graph);

    return Fl::run();
}


